
import sys, os
sys.path.append(os.path.abspath(os.path.dirname(sys.argv[0]))+"/../") 
            
from Crypto.Cipher import AES            
import hmac, sha
import struct

class PacketUtil:
    @staticmethod
    def CreateAESObject(key, keySize, iv, blocksize):
        return AES.new(key[:keySize], AES.MODE_CBC, iv[:blocksize])
        
    @staticmethod
    def EncryptAndMACPacket(seqid, packetData, aesObj, macKey):
        return aesObj.encrypt(packetData) + hmac.new(macKey, PacketUtil.GetMacData(seqid, packetData), sha.sha).digest()
    
    @staticmethod
    def GetMacData(seqid, packet):
        return struct.pack('>L', seqid) + packet 

    @staticmethod
    def DecryptPacket(data, aesObj):
        return aesObj.decrypt(data)
        
    @staticmethod
    def VerifyMAC(macData, seqid, packetData, macKey):
        return macData == hmac.new(macKey, PacketUtil.GetMacData(seqid, packetData), sha.sha).digest()


if __name__=="__main__":
    import base64
        
    # Testing CreateAESObject
    key = base64.decodestring("KZ9tLNwsMbicA89AsxA14mSS1PLas+L2w4Dhua5tYML6Ez1WuFLKBQ==")
    iv = base64.decodestring("ufkhLpjBjWelSQeseoih2m582kf7BKTUQnRciXrWm3TrgLnqDkpFhg==")
    awesomeStrEnc = base64.decodestring("MB6cJk5nHA3Epst1TOVF0g==")
    aes = PacketUtil.CreateAESObject(key, 16, iv, 16)
    assert aes.encrypt("432 is awesome! ") == awesomeStrEnc, "PacketUtil::CreateAESObject error"

    # Testing EncryptAndMACPacket
    packetData = base64.decodestring("AAAAHAoFAAAADHNzaC11c2VyYXV0aFJjH4X/tq6AbjQ=")
    macKey = base64.decodestring("BG6lqFkdxM37dottA2GGjoosJAoAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA\nAAAAAAAAAA==")
    p = base64.decodestring("lB+zDshjk2gd+/A6BAOv9afpHmS5/QwnQSshJ0CrWfp1m36ew1k6//q7M3/iqMmXTcMXsQ==")
    assert PacketUtil.EncryptAndMACPacket(3, packetData, aes, macKey) == p, "PacketUtil::EncryptAndMACPacket error"
    
    # Testing DecryptPacket
    aes = PacketUtil.CreateAESObject(key, 16, iv, 16)
    assert PacketUtil.DecryptPacket(awesomeStrEnc, aes) == "432 is awesome! ", "PacketUtil::DecryptPacket error"
    
    # Testing VerifyMAC
    macData = base64.decodestring("LCsZkxQrVg5QxvUBn8fkI//+p0g=")
    seqid = 18
    packetData = base64.decodestring("AAAADAZhAAAAAI4D6JDqng==")
    macKey = base64.decodestring("ZFtxCD8xmvmYx29M9rDa4msVc4kAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA\nAAAAAAAAAA==")
    assert PacketUtil.VerifyMAC(macData, seqid, packetData, macKey), "PacketUtil::VerifyMAC error"
    
    
    print "No errors"
    
    
