import sys, os, math
sys.path.append(os.path.abspath(os.path.dirname(sys.argv[0]))+"/../") 

from PythonSSH.Random import Random
from IntUtil import IntUtil


from cPickle import dumps, loads
import base64
import zlib

class RSA:
    """
    
    >>> (public, private) = RSA.GeneratePublicAndPrivateKeys(100)
    >>> private['p']*private['q'] == public['n']
    True
    >>> encrypted = RSA.EncryptInt(2387945734, private['d'], private['p']*private['q'])
    >>> decrypted = RSA.DecryptInt(encrypted, public['e'], public['n'])
    >>> decrypted == 2387945734
    True
    """
    
    
    @staticmethod
    def GeneratePublicAndPrivateKeys(bits):
        """
        Generate public and private keys by first generating p and q 
        and then generating e and d       
        
        Returns (public keys, private keys)
        """
        (p, q) = RSA.GeneratePandQ(bits)
        (e, d) = RSA.GenerateKeys(p, q, bits)

        return ({'e': e, 'n': p*q}, {'d': d, 'p': p, 'q': q})
   
    @staticmethod
    def GenerateKeys(p, q, bits):
        """
        Given p and q, generate the public key (e) and private key (d) and return
        (e, d)
        
        @param bits: The number of bits to make p and q
        
        Note: Make the private key (d) positive. This can be done by 
        adding phi_n to d as many times as necessary until d is positive.
        
        Note: Make sure e and phi_n are relatively prime
        
        (assuming p*q = n, phi(n) is (p-1)*(q-1) aka Euler's totient function:
        http://en.wikipedia.org/wiki/Totient_function )
        
        Note: Make sure e and d are multiplicative inverses
        """
        pass #STUDENT WORK GOES HERE
        
    @staticmethod
    def GeneratePandQ(bits):
        """
        Return the pair (p,q) where p and q are primes. You must ensure that
        p and q are not the same value
        
        @param bits: The number of bits to make p and q
        
        """
        pass #STUDENT WORK GOES HERE

    @staticmethod
    def EncryptInt(message, e, n):
        """
        The message, e and n do not require any preprocessing
        """
        pass #STUDENT WORK GOES HERE

    @staticmethod
    def DecryptInt(ciphertext, d, n):
        """
        The message, e and n do not require any preprocessing
        """
        pass #STUDENT WORK GOES HERE

    @staticmethod
    def SignInt(message, key, n):
        return RSA.EncryptInt(message, key, n)

    @staticmethod
    def VerifyInt(signed, key, n):
        return RSA.DecryptInt(signed, key, n)





class RSAUtil:
    
    @staticmethod
    def GetPrime(bits):
        """
        Return a prime number that has b random bits (fine to round up to bytes)

        This can be done by generating random numbers until a prime number
        is found
        
        (You can speed up this function by ensuring that (the number
        you chose) and (the product of the first 10 or so primes) are
        relatively prime (gcd == 1) before calling IsPrime. This is optional.)
        
        >>> for bits in range(80, 130, 27):
        ...     p = RSAUtil.GetPrime(bits)
        ...     RSAUtil.IsPrime(p) and IntUtil.bits2bytes(math.log(p)/math.log(2)) == IntUtil.bits2bytes(bits)
        True
        True
        >>> n = RSAUtil.GetPrime(512)
        """
        pass #STUDENT WORK GOES HERE
    
    
    @staticmethod
    def ExtendedEuclidean(a,b):
        """
        Implement the Extended Euclidean algorithm:
        
        Given a and b, the extended euclidean algorithm finds x and y s.t.
            a*x + b*y = gcd(a,b)
            
        Return the tuple
            (z, x, y) 
        where z is the gcd of a and b and x and y are the x and y above.
        
        http://en.wikipedia.org/wiki/Extended_Euclidean_algorithm
        
        >>> RSAUtil.ExtendedEuclidean(40,30)
        (10, 1, -1)
        >>> RSAUtil.ExtendedEuclidean(352,168)
        (8, -10, 21)
        >>> RSAUtil.ExtendedEuclidean(120,23)
        (1, -9, 47)
        >>> RSAUtil.ExtendedEuclidean(3458,4864)        
        (38, -45, 32)
        """
        pass #STUDENT WORK GOES HERE

    
    
        


    @staticmethod
    def gcd(p,q):
        """
        Implement the GCD algorithm
        
        http://en.wikipedia.org/wiki/Greatest_common_divisor
        
        >>> RSAUtil.gcd(5, 2)
        1
        >>> RSAUtil.gcd(20, 4)
        4
        >>> RSAUtil.gcd(35, 20)
        5
        >>> RSAUtil.gcd(42, 56)
        14
        >>> RSAUtil.gcd(56, 42)
        14
        """
        
        pass #STUDENT WORK GOES HERE             
    
    
    
    
    @staticmethod
    def IsPrime(n):
        """
        Returns True if number is prime, else False
        """
        return RSAUtil.MillerRabin(n)

        
        
        
    @staticmethod
    def MillerRabin(n, k = 50):
        """
        Implement the Miller-Rabin algorithm (described in Appendix
        A of the handout)
        
        Return True if (probably) prime
        Return False if definitely composite

        >>> RSAUtil.MillerRabin(4)
        False
        >>> RSAUtil.MillerRabin(516119616549881)
        True
        >>> RSAUtil.MillerRabin(516119616549887)
        False
        >>> RSAUtil.MillerRabin(750939670750127524461407364479090229319)
        True
        """
        

        pass #STUDENT WORK GOES HERE             
        
    @staticmethod
    def MillerRabinWitness(a,n):
        """
        Implement the Miller-Rabin (witness) algorithm (described in Appendix
        A of the handout)        
        
        Return True if definitely composite
        Return False if possibly prime
        
        >>> RSAUtil.MillerRabinWitness(174,221)
        False
        >>> RSAUtil.MillerRabinWitness(137,221)
        True
        """

        pass #STUDENT WORK GOES HERE             
        
        

    @staticmethod
    def fromBin(s):
        """
        
        >>> RSAUtil.fromBin('0')
        0
        >>> RSAUtil.fromBin('1')
        1
        >>> RSAUtil.fromBin('10')
        2
        >>> RSAUtil.fromBin('11')
        3
        >>> RSAUtil.fromBin('100')
        4
        >>> RSAUtil.fromBin('1110110011')
        947
        """
        
        # reduce uses a pair (x,y) where x is the current accumulated value
        #   and y is the position in the string (so 2^y is computed)
        return reduce(lambda a, b: (a[0]+int(b)*2**a[1], a[1]+1), reversed(s), (0,0))[0]

    @staticmethod
    def toBin(n):
        """
        >>> RSAUtil.toBin(0)
        '0'
        >>> RSAUtil.toBin(1)
        '1'
        >>> RSAUtil.toBin(2)
        '10'
        >>> RSAUtil.toBin(3)
        '11'
        >>> RSAUtil.toBin(4)
        '100'
        >>> RSAUtil.toBin(947)
        '1110110011'
        """
        l = []
        
        if n == 0:
            return '0'
        
        while n >= 1:
            l.append(n%2)
            n = n//2
            
        l.reverse()
        return "".join(map(str, l))
    



if __name__ == "__main__":
    import doctest
    doctest.testmod()    
