

from PythonSSH.Server.SSHShell import SSHShell
# START_TA_CODE
from PythonSSH.Fun.RSS import feedparser
# END_TA_CODE

class MySSHShell(SSHShell):

    def __init__(self, *args):
        SSHShell.__init__(self, *args);
        self.mydict = {}

    def connectionMade(self): 
        SSHShell.connectionMade(self)
        self.write("Welcome to my test SSH server.")
        self.nextLine()
        self.do_help()
        self.showPrompt()

    def do_echo(self, *args):
        "Echo a string. Usage: echo my line of text"
        self.write(" ".join(args))
        self.nextLine()

    def do_quit(self):
        "Ends your session. Usage: quit"
        self.write("Thanks for playing!")
        self.nextLine()
        self.terminal.loseConnection()

    def do_clear(self):
        "Clears the screen. Usage: clear"
        self.terminal.reset()

    def do_mult(self, x, y):
        """
        Multiple two numbers together. Usage: mult x y
        ...try large numbers
        """
        self.write(int(x)*int(y))
        self.nextLine()
        
    def fact(self, n):
        if n <= 1:
            return 1
        else:
            return n*self.fact(n-1)
    
    def do_fact(self, n):
        "Computes the factorial of a number. Usage: fact n"
        self.write(self.fact(int(n)))
        self.nextLine()

    def do_loop(self, n):
        "Loops from 1 to n (n is positive). Usage: loop n"
        for i in range(1, int(n)+1):
            self.write(i)
            self.nextLine()
            
    def do_max(self, *args):
        "Find the max of a list of integers. Usage: max n1 n2 n3 n4 ..."
        max = None
        for i in args:
            i = int(i)
            if max is None or i > max:
                max = i
        self.write(max)
        self.nextLine()
        
    def do_max_lambda(self, *args):
        "The same as max, but uses python's map() and reduce() and lambda functions. Usage: max_reduce n1 n2 n3 n4 ..."
        self.write(reduce(lambda x,y: (x > y and [x] or [y])[0], map(int, args)))
        self.nextLine()

        
    def do_sum(self, *args):
        "Find the sum of a list of integers. Usage sum n1 n2 n3 n4 ..."
        sum = 0
        for i in args:
            sum += int(i)
        self.write(str(sum))
        self.nextLine()
 
        
    def do_sum_lambda(self, *args):
        "Find the sum of a list of integers using Python's lambda functions. Usage sum n1 n2 n3 n4 ..."
        self.write(reduce(lambda x,y: x+y, map(int, args),0))
        self.nextLine()
        
    def do_sort(self, *args):
        "Sort a list of integers. Usage sum n1 n2 n3 n4 ..."
        self.write(" ".join(map(str, sorted(map(int, args)))))
        self.nextLine()
        
        
    def do_dict_put(self, key, value):
        "Add an item to a dictionary. Usage: dict_put key value (store a value in the dict)"
        self.mydict[key] = value
        self.write("Added '"+key+"' to dictionary")
        self.nextLine()

    def do_dict_get(self, key):
        "Add an item to a dictionary. Usage: dict_get key (print out the key's value)"
        self.write(self.mydict[key])
        self.nextLine()

    def do_dict_test(self, key):
        "Add an item to a dictionary. Usage: dict_test key (print out True/False == if a key exists)"
        self.write(str(self.mydict.has_key(key)))
        self.nextLine()

    def do_dict_list(self):
        "Add an item to a dictionary. Usage: dict_list (print out all key/value pairs in dict)"
        for key, value in self.mydict.iteritems():
            self.write(key+" = "+value)
            self.nextLine()

    def do_myhelp(self):
        "Simply call the superclass's help function"
        SSHShell.do_help(self)


    def do_rss(self):
        "Retreives an RSS feed and print out the titles. Usage: rss"
        d = feedparser.parse("http://www.nytimes.com/services/xml/rss/nyt/HomePage.xml")
        for item in d['entries']:
            self.write(item.title.encode('ascii','replace'))
            self.nextLine()
