import sys, os
sys.path.append(os.path.abspath(os.path.dirname(sys.argv[0]))+"/../../") 

from twisted.cred import portal, checkers, credentials
from twisted.conch import error, avatar, recvline, interfaces as conchinterfaces
from twisted.conch.ssh import factory, userauth, connection, keys, session, common
from twisted.conch.insults import insults
from twisted.application import service, internet
from twisted.internet import protocol
from twisted.internet import reactor
from twisted.python import log
from twisted.conch.ssh import transport

from zope import interface
from zope.interface import implements
import os
import re
import sys
import math

import pickle
from Crypto import Util

#from PythonSSH.Server.ServerCommandTransport import ServerCommandTransport
from PythonSSH.Server.MyServerCommandTransport import MyServerCommandTransport

from PythonSSH.Server.MySSHShell import MySSHShell
from PythonSSH.RSA import RSA
from PythonSSH.IntUtil import IntUtil

class SSHDemoAvatar(avatar.ConchUser):
    implements(conchinterfaces.ISession)

    def __init__(self, username):
        avatar.ConchUser.__init__(self)
        self.username = username
        self.channelLookup.update({'session':session.SSHSession})
        self.pty = None
    
    def openShell(self, protocol):
        serverProtocol = insults.ServerProtocol(MySSHShell, self)
        serverProtocol.makeConnection(protocol)
        protocol.makeConnection(session.wrapProtocol(serverProtocol))


    def getPty(self, terminal, windowSize, attrs):
		return self.pty

    def eofReceived(self):
        if self.pty:
            self.pty.closeStdin()


    def closed(self):
        pass

class SSHDemoRealm:
    implements(portal.IRealm)

    def requestAvatar(self, avatarId, mind, *interfaces):
        if conchinterfaces.IConchUser in interfaces:
            #return interfaces[0], SSHDemoAvatar(avatarId), lambda: None
            return interfaces[0], SSHDemoAvatar(avatarId), lambda: None
        else:
            raise Exception, "No supported interfaces found."

def getRSAKeys():
    if not (os.path.exists('public.key') and os.path.exists('private.key')):
        # generate a RSA keypair
        print "Generating RSA keypair..."
        KEY_LENGTH = 1024
        (publicKey, privateKey) = RSA.GeneratePublicAndPrivateKeys(KEY_LENGTH)
        publicKeyString = pickle.dumps(publicKey)
        privateKeyString = pickle.dumps(privateKey)
        # save keys for next time
        file('public.key', 'w+b').write(publicKeyString)
        file('private.key', 'w+b').write(privateKeyString)
        print "done."
    else:
        publicKey = pickle.loads(file('public.key').read())
        privateKey =  pickle.loads(file('private.key').read())
    return publicKey, privateKey



class ServerCommandFactory(factory.SSHFactory):
    def __init__(self):
        pass
        #self.factory = factory.SSHFactory.factory
        
    def buildProtocol(self, addr):
        t = MyServerCommandTransport()
        t.supportedPublicKeys = self.privateKeys.keys()
        if not self.primes:
            log.msg('disabling diffie-hellman-group-exchange because we '
                    'cannot find moduli file')
            ske = t.supportedKeyExchanges[:]
            try:
                ske.remove('diffie-hellman-group-exchange-sha1')
            except ValueError:
                pass
            t.supportedKeyExchanges = ske
        t.factory = self
        #protocol = transport.SSHServerTransport()
        return t
        
class FakePublicKey:
    """
    Class that mimics the conch public key class but uses our custom
    RSA class
    """
    def __init__(self, publicKey):
        self.e = publicKey['e']
        self.n = publicKey['n']
        
        
    def pickle(self):
        #print "e = ", self.e
        #print "n = ", self.n
        #print "bits to bytes = " , IntUtil.bits2bytes(math.log(self.n)/math.log(2))
        return (common.NS('ssh-rsa') + common.MP(self.e) +
                    common.MP(self.n))
                    
        #return pickle.dumps({'e': self.e, 'n': self.n})
    

class FakePrivateKey:
    """
    Class that mimics the conch public key class but uses our custom
    RSA class
    """
    def __init__(self, privateKey):
        self.d = privateKey['d']
        self.p = privateKey['p']
        self.q = privateKey['q']
        self.n = self.p * self.q

    def sign(self, data):
        return RSA.Sign(data, self.d, self.n)
        
    def sign2(self, data):
        digest = keys.pkcs1Digest(data, 1024/8)
        #print "Util.number.bytes_to_long(digest) = ", Util.number.bytes_to_long(digest)
        signature = RSA.SignInt(Util.number.bytes_to_long(digest), self.d, self.n)
        #print "signature = ", signature
        ret = common.NS(Util.number.long_to_bytes(signature))
        #print "ret = ", ret
        return common.NS('ssh-rsa') + ret

if __name__ == "__main__":
    if len(sys.argv) < 2:
        print "Usage: python MyServer.py port"
        sys.exit(0)    
        
    log.startLogging(sys.stdout)

    sshFactory = ServerCommandFactory();
    sshFactory.portal = portal.Portal(SSHDemoRealm())
    users = {'aaa': 'bbb', 'username':'password'}
    sshFactory.portal.registerChecker(
         checkers.InMemoryUsernamePasswordDatabaseDontUse(**users))

    #pubKeyString, privKeyString = getRSAKeys()

    (publickey, privateKey) = getRSAKeys()

    sshFactory.publicKeys = {'ssh-rsa': FakePublicKey(publickey)}
    #sshFactory.privateKeys = {'ssh-rsa': FakePrivateKey(privateKey)}
    sshFactory.myPrivateKeys = {'ssh-rsa': FakePrivateKey(privateKey)}
    sshFactory.getPrivateKeys = lambda: {'ssh-rsa': None}
	
    from twisted.internet import reactor
    reactor.listenTCP(int(sys.argv[1]), sshFactory)
    reactor.run() 
